#pragma once

#include <cstdint>
#include <iterator>

template <typename T>
struct Counter
{
  using difference_type = std::ptrdiff_t;

  struct EmptyRef {
    auto operator=(T const&) const -> void {}
    auto operator=(T &&    ) const -> void {}
  };

  std::size_t n;

  Counter() : n{0} {}
  Counter(std::size_t n) : n{n} {}

  // prefix increment
  auto operator++() -> Counter&
  {
    n++;
    return *this;
  }

  // postfix increment
  auto operator++(int) -> Counter
  {
    return n++;
  }

  auto operator*() const -> EmptyRef
  {
    return {};
  }

  auto operator-(Counter rhs) const -> difference_type
  {
    return difference_type{n} - difference_type{rhs};
  }

  operator std::size_t() const
  {
    return n;
  }
};

static_assert(std::output_iterator<Counter<int>, int>);
